/**********************************************************
kkDesktopInfo for E2 v0.1b

Written  by Strelok Vlad Kolerts
5/12/2009
Email: admin@mobsk.org.ua
ICQ: 497123239
***********************************************************/

//#include "QDaemonApp.h"
#include <ZApplication.h>
#include <ZKbMainWidget.h>
#include <UTIL_MemInfo.h>
#include <SETUP_API.h>
#include <ZConfig.h>
#include <KString.h>

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <qcopchannel_qws.h>
#include <qpixmapcache.h>
#include <qtextstream.h>
#include <qwallpaper.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qfile.h>

class QDaemonApp : public ZApplication 
{
  Q_OBJECT
  public:

    QDaemonApp(int argc, char **argv);

    virtual ~QDaemonApp();
  protected slots:
    virtual void slotShutdown(); 
    virtual void slotQuickQuit();
    virtual void slotRaise();
};

QDaemonApp::QDaemonApp(int argc, char **argv) : ZApplication(argc, argv) 
{
}

QDaemonApp::~QDaemonApp()
{
}

void QDaemonApp::slotShutdown()
{
    processEvents();
}
void QDaemonApp::slotQuickQuit()
{
    processEvents();
}
void QDaemonApp::slotRaise()
{
    processEvents();
}

extern "C" int UTIL_GetIdleFocus();
extern "C" int UTIL_GetChargeStatus();
extern "C" int UTIL_GetBatteryLevel();

#include <vector>

struct registry_structure
{
	QString name;
	QString action;
	QString text;
	QString line;
	int x;
	int y;
	int w;
	int h;
	int size;
	int align;
	QString color;
	QString bcolor;
};

class kDesktopInfo : public ZWidget
{
    Q_OBJECT

public:
    kDesktopInfo();
    ~kDesktopInfo();
	
	std::vector<registry_structure> data;
	
	QPainter	p;
	///QPixmap 	bg_buf;///
	QPixmap*	bg_buf;///
	QPixmap 	p_bg;
	QPixmap 	pm;
	QWallpaper	desktop;
	
	QFont	font;
	QTimer*	timer;
	SETUP_API* api;
	UTIL_MemInfo mem;
	QCopChannel *cop;
	//QCopChannel *cop_desktop;
	
	QString path, theme;
	QString wallpaper;
	//QString test;
	
	int timer_delay;
	int labels_c;
	bool my_focus, refresh_unidle, debug;
	
	QString lng_week[7], lng_fweek[7], lng_month[12];
	QString	lng_charge, lng_charge_complete;
	
	void paintMessage(QString text, int x, int y, int w, int h, int align, int size, QString font_col, QString border_col);
	void paintMultiLineMessage(QString text, int x, int y, int w, int h, int align, int size, QString font_col, QString border_col);
	void drawImage(int i, QString file, int x, int y, int w, int h);
	void drawDinamicImage(QString file, int x, int y, int w, int h);
	
	QString getText(int i);
	
	QString getDate(QString line);
	QString getTime(QString line);
	QString getCpuHz(QString line);
	QString getRamValues(QString line);
	QString getFile(QString line);
	QString getCmd(QString line);
	QString getChargeStat();
	
	void readConfig();
	void readIni();
	void refresh();
	void clearBg();
	double tof(long size);
	//void save_buf();
	void Debug(QString mes);
	
public slots:

	void changeInfo();
	void slotQCopReceived(const QCString&);
};

kDesktopInfo::kDesktopInfo() : 
    ZWidget(NULL, "ZMainWidget", 0)
{
	setBackgroundMode(NoBackground);
	//setFullScreenMode(240, 320);
	resize(240, 320);
	
	QPixmapCache::setCacheLimit(2048);
	
	my_focus=0;

	path=kGetPath(QString(qApp->argv()[0]));
	system("rm /tmp/.cmd_tmp");
	
	readConfig();
	readIni();
	
	font=QFont(NULL, 10, 25, false);

	api= new SETUP_API();
	api->getWallpaper(wallpaper);

	p_bg.load(wallpaper);
	
	bg_buf = new QPixmap(240,320);///
	bg_buf->load(wallpaper);///
	
	bg_buf=desktop.wallpaper();///
	///bg_buf=p_bg;///
	
	cop = new QCopChannel("kDesktopInfo", this);
	connect(cop, SIGNAL(received(const QCString&, const QByteArray&)), this, SLOT(slotQCopReceived(const QCString&)));
	
	/*cop_desktop = new QCopChannel("/app/phone/msg/idlestat", this);
	connect(cop_desktop, SIGNAL(received(const QCString&, const QByteArray&)), this, SLOT(slotQCopReceived(const QCString&)));*/
	
	///p.begin( &bg_buf ,this);///
	p.begin( bg_buf ,this);///
	
	timer = new QTimer(this);
	connect( timer, SIGNAL(timeout()), SLOT(changeInfo()) );
	timer->start(timer_delay);
	//QCopChannel::send("/app/phone/msg/idlestat", "idleScrFocusStatMsg");
	QTimer::singleShot(1, this, SLOT(hide()));
	QTimer::singleShot(1, this, SLOT(changeInfo()));
}
kDesktopInfo::~kDesktopInfo()
{

}
/*void kDesktopInfo::save_buf()
{ 
	//"/tmp/qdesktop"
	//"/ezx_user/download/appwrite/setup/theme/User_0.chm"
}*/


void kDesktopInfo::clearBg()
{
	/**api->getWallpaper(wallpaper);
	if(wallpaper!="/tmp/qdesktop")///
	{
		Debug("reload wallpaper");
		p_bg.load(wallpaper);
	}*/
	p.drawPixmap(0, 0, p_bg);
}
void kDesktopInfo::refresh()
{
	/**if(bg_buf.save("/tmp/qdesktop", "PNG"))
	{
		SETUP_API* api= new SETUP_API();	
		api->setWallpaper("/tmp/qdesktop");
	}
	else
		Debug("cant save buffer");*/
	show();
	hide();

	Debug("desktop hard update");
	
	/*if(bg_buf.save("/tmp/qdesktop", "PNG"))
	{
		save_buf();
		system(path+"/qcop_send /AM/Launcher notifyUpdateHomeView");
	}
	else
		Debug("cant save buffer");*/
}
// ------------------------------------------
void kDesktopInfo::changeInfo()
{
	if(UTIL_GetIdleFocus() || QCopChannel::isRegistered("kLaunchInFocus") || refresh_unidle)
	{
		clearBg();
		//Debug("start paint");
		//test+=".";
		/*paintBorderMessage(getDate("%w %d %c %y"), 5, 68, -1, -1, Qt::AlignLeft, 17, QColor("#ffffff"), QColor("#000000"));
		paintBorderMessage(getTime("%h:%m"), 10, 240, Qt::AlignLeft, 60, QColor("#ffffff"), QColor("#000000"));

		paintMessage(getChargeStat(), 190, 22, Qt::AlignRight, 9, QColor(155,0,0));
		//paintMessage(test, 5, 160, Qt::AlignLeft, 9, QColor(255,0,0));
		//Debug(QString("Phone status:%1").arg(isDesktop()));*/
		
		for(int i=0;i<labels_c;i++)
		{
			Debug("\nAction:"+data[i].action);
			if(data[i].action=="drawMessage")
				paintMessage(getText(i), data[i].x, data[i].y, data[i].w, data[i].h, data[i].align, data[i].size, data[i].color, data[i].bcolor);
			if(data[i].action=="drawMultiLineMessage")
				paintMultiLineMessage(getText(i), data[i].x, data[i].y, data[i].w, data[i].h, data[i].align, data[i].size, data[i].color, data[i].bcolor);
			else if(data[i].action=="drawImage")
				drawImage(i, data[i].text, data[i].x, data[i].y, data[i].w, data[i].h);
			else if(data[i].action=="drawDinamicImage")
				drawDinamicImage(data[i].text, data[i].x, data[i].y, data[i].w, data[i].h);
		}
	
		Debug("desktop updated");
	}
	
	if(UTIL_GetIdleFocus() || QCopChannel::isRegistered("kLaunchInFocus"))
		refresh();
}

QString kDesktopInfo::getText(int i)
{
	Debug("GetText::"+data[i].text+" "+data[i].line);
	QString text="";
	if(data[i].text=="Time")
		text=getTime(data[i].line);
	else if(data[i].text=="Date")
		text=getDate(data[i].line);
	else if(data[i].text=="ChargeStatus")
		text=getChargeStat();
	else if(data[i].text=="Cpu")
		text=getCpuHz(data[i].line);
	else if(data[i].text=="RamInfo")
		text=getRamValues(data[i].line);
	else if(data[i].text=="Cmd")
		text=getCmd(data[i].line);
	else if(data[i].text=="File")
		text=getFile(data[i].line);
	else
		text=data[i].text;
	Debug("GetText::"+text);
	return text;
}

// ------------------------------------------

void kDesktopInfo::paintMessage(QString text, int x, int y, int w, int h, int align, int size, QString font_col, QString border_col)
{
	if(text!="")
	{
		QColor fontCol=QColor(font_col);
		QColor borderCol=QColor(border_col);
		font.setPixelSize(size);
		p.setFont(font);
		if(w==-1)	w=240-x;
		if(h==-1)	h=size+5;
		
		if(border_col!="")
		{
			p.setPen(borderCol);
			p.drawText(x-1, y-1, w+1, h+1, align, text, -1);
			p.drawText(x-1, y+1, w+1, h-1, align, text, -1);
			p.drawText(x+1, y-1, w-1, h+1, align, text, -1);
			p.drawText(x+1, y+1, w-1, h-1, align, text, -1);
		}
		p.setPen(fontCol);
		p.drawText(x, y, w, h, align, text, -1);
	}
	Debug("drawMessage::ok");
}

void kDesktopInfo::paintMultiLineMessage(QString text, int x, int y, int w, int h, int align, int size, QString font_col, QString border_col)
{
	if(text!="")
	{
		QColor fontCol=QColor(font_col);
		QColor borderCol=QColor(border_col);
		font.setPixelSize(size);
		p.setFont(font);
		if(w==-1)	w=240-x;
		if(h==-1)	h=size+5;
		
		int pos = text.find("\n");
		while(pos!=-1)
		{
			if(border_col!="")
			{
				p.setPen(borderCol);
				p.drawText(x-1, y-1, w+1, h+1, align, text.left(pos), -1);
				p.drawText(x-1, y+1, w+1, h-1, align, text.left(pos), -1);
				p.drawText(x+1, y-1, w-1, h+1, align, text.left(pos), -1);
				p.drawText(x+1, y+1, w-1, h-1, align, text.left(pos), -1);
			}
			p.setPen(fontCol);
			p.drawText(x, y, w, h, align, text, -1);
			
			text.remove(0, pos+2);
			pos=text.find("\n");
			y+=h;
		}
		
		if(border_col!="")
		{
			p.setPen(borderCol);
			p.drawText(x-1, y-1, w+1, h+1, align, text, -1);
			p.drawText(x-1, y+1, w+1, h-1, align, text, -1);
			p.drawText(x+1, y-1, w-1, h+1, align, text, -1);
			p.drawText(x+1, y+1, w-1, h-1, align, text, -1);
		}
		p.setPen(fontCol);
		p.drawText(x, y, w, h, align, text, -1);
	}
	Debug("drawMultiLineMessage::ok");
}

void kDesktopInfo::drawImage(int i, QString file, int x, int y, int w, int h)
{
	if(file.left(1)!="/")
		file=path+"themes/img/"+file;
	if(!QPixmapCache::find(data[i].name, pm))
	{
		pm.load(file);
		if(w!=-1 && h!=-1)
			pm.convertFromImage(pm.convertToImage().smoothScale(w, h));
		QPixmapCache::insert(data[i].name, pm);
	}
	p.drawPixmap(x, y, pm);
	Debug("drawImage::ok");
}


void kDesktopInfo::drawDinamicImage(QString file, int x, int y, int w, int h)
{
	if(file.left(1)!="/")
		file=path+"themes/img/"+file;
	
	pm.load(file);
	if(w!=-1 && h!=-1)
		pm.convertFromImage(pm.convertToImage().smoothScale(w, h));
	
	p.drawPixmap(x, y, pm);
	Debug("drawDinamicImage::ok");
}

// ------------------------------------------

QString kDesktopInfo::getTime(QString line)
{// %h - hour(int), %m - min(int), %s - sec(int), %H - hour12(int), %i - ampm(str)
	QString ampm, mm, ss;
	QDateTime date_time;
	QTime time=date_time.currentDateTime().time();
	int h=time.hour();
	int m=time.minute();
	int s=time.second();
		
	int hh=h;
	if(m<10)	mm=QString("0%1").arg(m);	else mm=QString("%1").arg(m);
	if(s<10)	ss=QString("0%1").arg(s);	else ss=QString("%1").arg(s);

	if(hh<=12)
		ampm="am";
	else
	{
		hh=hh-12;
		ampm="pm";
	}
	if(hh==0)
		ampm="pm";
	
	int i=0;int len=strlen(line);
	QString tmp;
	QString text="";
	while(i<len)
	{
		tmp=line.mid(i,1);
		if(tmp!="%")
			text+=tmp;
		else
		{
			i++;
			tmp=line.mid(i,1);
			if(tmp=="h")
				text+=QString::number(h);
			else if(tmp=="m")
				text+=mm;
			else if(tmp=="s")
				text+=ss;
			else if(tmp=="H")
				text+=QString::number(hh);
			else if(tmp=="i")
				text+=ampm;
			else 
				text+="%"+tmp;
		}
		i++;
	}
	return text;
}

QString kDesktopInfo::getDate(QString line)
{// %d - day(int), %m - month(int), %y - year(int), %w - day of week(str), %M - month(str), %c - month cut(str)
	QDateTime date_time;
	QDate date=date_time.currentDateTime().date();
	int w=date.dayOfWeek();
	int d=date.day();
	int m=date.month();
	int y=date.year();
	
	int i=0;int len=strlen(line);
	QString tmp;
	QString text="";			
	while(i<len)
	{
		tmp=line.mid(i,1);
		if(tmp!="%")
		{
			text+=tmp;
		}
		else
		{
			i++;
			tmp=line.mid(i,1);
			if(tmp=="d")
				text+=QString::number(d);
			else if(tmp=="m")
				text+=QString::number(m);
			else if(tmp=="y")
				text+=QString::number(y);
			else if(tmp=="w")
				text+=lng_week[w-1];
			else if(tmp=="W")
				text+=lng_fweek[w-1];
			else if(tmp=="M")
				text+=lng_month[m-1];
			else if(tmp=="c")
				text+=lng_month[m-1].left(3);
			else 
				text+="%"+tmp;
		}
		i++;
	}
	return text;
}

QString kDesktopInfo::getRamValues(QString line)
{//	%u - ram total(Kb), %f - ram avalible(Kb), %s - swap total(Kb), %F - swap free(Kb), %p - ram free(%), %P - swap free(%)
	
	mem.refresh();
	
	int i=0;int len=strlen(line);
	QString tmp;
	QString text="";
	
	while(i<len)
	{
		tmp=line.mid(i,1);
		if(tmp!="%")
			text+=tmp;
		else
		{
			i++;
			tmp=line.mid(i,1);
			if(tmp=="u")
				text+=QString("%1").arg(tof(mem.memTotal()));
			else if(tmp=="f")
				text+=QString("%1").arg(tof(mem.cached()));
			else if(tmp=="s")
				text+=QString("%1").arg(tof(mem.swapTotal()));
			else if(tmp=="F")
				text+=QString("%1").arg(tof(mem.swapFree()));
			else if(tmp=="p")
				text+=QString("%1").arg((int)(tof(mem.cached())/tof(mem.memTotal())*100));
			else if(tmp=="P")
				text+=QString("%1").arg((int)(tof(mem.swapFree())/tof(mem.swapTotal())*100));
			else 
				text+="%"+tmp;
		}
		i++;
	}
	return text;
}

QString kDesktopInfo::getCpuHz(QString line)
{
	QString text;
	QFile cpufile("/proc/cpuinfo");
	if(cpufile.open( IO_ReadOnly )) 
	{
		QTextStream stream( &cpufile );
		stream.readLine();
		text = stream.readLine();
		cpufile.close();
	}
	text=text.right(6);
	text=text.left(3);
	
	if(line=="")
		line="%1";
	return line.arg(text);
}

QString kDesktopInfo::getCmd(QString line)
{
	system(line+" > /tmp/.cmd_tmp");
	QFile file("/tmp/.cmd_tmp");
	if( file.open(IO_ReadOnly) )		
	{
		QTextStream stream( &file );
		stream.setEncoding ( QTextStream::UnicodeUTF8 );
		return stream.read();
	}
	else
		return "<Error>";
}

QString kDesktopInfo::getFile(QString line)
{
	if(line.left(1)!="/")
		line=path+"plugins/"+line;
	QFile file(line);
	if( file.open(IO_ReadOnly) )		
	{
		QTextStream stream( &file );
		stream.setEncoding ( QTextStream::UnicodeUTF8 );
		return stream.read();
	}
	else
		return "<Error>";
}

QString kDesktopInfo::getChargeStat()
{
	if(UTIL_GetChargeStatus()==1)
	{
		if(UTIL_GetBatteryLevel()==5)
			return lng_charge_complete;
		else
			return lng_charge;
	}
	else
		return NULL;
}
// --------------------------------------------------------

void kDesktopInfo::readIni()
{
	registry_structure tmp;
	QStringList labels;
	data.clear();
	
	QFile them(path+"themes/"+theme+".theme");
	if(them.exists())
	{
		ZConfig cfg(path+"themes/"+theme+".theme", true);
		cfg.getGroupsKeyList(labels);
		QString plugins=cfg.readEntry("Plugins", "StartLine", "");
		if(plugins!="")
			plugins+="|";
		labels_c=labels.count();
		Debug("Start reading labels..");
		for(int i=0;i<(int)labels.count();i++)
		{
			/*if(labels[i]!=QString("Plugins"))
			{*/
				tmp.name=labels[i];
				tmp.action=cfg.readEntry(labels[i], "Action", "");
				tmp.text=cfg.readEntry(labels[i], "Text", "");
				tmp.line=cfg.readEntry(labels[i], "Line", "");
				tmp.x=cfg.readNumEntry(labels[i], "PosX", 0);
				tmp.y=cfg.readNumEntry(labels[i], "PosY", 0);
				tmp.w=cfg.readNumEntry(labels[i], "Width", -1);
				tmp.h=cfg.readNumEntry(labels[i], "Height", -1);
				tmp.size=cfg.readNumEntry(labels[i], "Size", 13);
				tmp.align=cfg.readNumEntry(labels[i], "Align", Qt::AlignLeft);
				tmp.color=cfg.readEntry(labels[i], "Color", "");
				tmp.bcolor=cfg.readEntry(labels[i], "BorderColor", "");
				
				if(tmp.action!="" && tmp.text!="")
				{
					Debug("\t"+tmp.name+": Action="+tmp.action+" Text="+tmp.text);
					data.push_back(tmp);
				}
				else
				{
					if(labels[i]!="Plugins")
						qDebug("Error at ["+labels[i]+"]");
					labels_c--;
				}
				/*
				tmp.action="";
				tmp.text="";
				tmp.line="";
				tmp.x=0;
				tmp.y=0;
				tmp.w=-1;
				tmp.h=-1;
				tmp.size=13;
				tmp.align=Qt::AlignLeft;
				tmp.color="";
				tmp.bcolor="";*/
			//}
		}
		cfg.flush();
		Debug("Finish reading labels");
		
		if(plugins!="")
		{
			QString cmd;
			int pos=plugins.find("|");
			while(pos!=-1)
			{
				pos = plugins.find("|");
				cmd=plugins.left(pos);
				if(cmd.left(1)!="/")
					cmd=path+"plugins/"+cmd;
				cmd+="&";
				if(cmd.right(3)==".sh" || cmd.right(4)==".lin")
				cmd="sh "+cmd;
				system(cmd);
				plugins.remove(0, pos+1);
			}
		}
	}
	else
	{
		qDebug("Error:can't find "+path+"themes/"+theme+".theme");
		qApp->quit();
	}
}

void kDesktopInfo::readConfig()
{
	ZConfig cfg(path+"kDesktopInfo.cfg", true);
	timer_delay=cfg.readNumEntry("Setting", "UpdateDelay", 5000);
	debug=cfg.readBoolEntry("Setting", "Debug", false);
	refresh_unidle=cfg.readBoolEntry("Setting", "RefreshUnidle", false);
	theme=cfg.readEntry("Setting", "Theme", "default");
	QString lang=cfg.readEntry("Setting", "Language", "english");
	cfg.flush();
	Debug("Language:"+lang+"\nTheme:"+theme);
	
	ZConfig lcfg(path+"lang/"+lang+".lng", true);
	QString line_fweek=lcfg.readEntry("Language", "FullWeekLine", "Monday|Tuesday|Wednesday|Thursday|Friday|Saturday|Sunday");
	QString line_week=lcfg.readEntry("Language", "WeekLine", "Mn|Tu|We|Th|Fr|Sa|Su");
	QString line_month=lcfg.readEntry("Language", "MonthLine", "January|February|March|April|May|June|July|August|September|October|November|December");
	lng_charge=lcfg.readEntry("Language", "Charge", "Charging..");
	lng_charge_complete=lcfg.readEntry("Language", "ChargeComlete", "Charge comlete");
	lcfg.flush();
	
/// Lang Week
	//UTF("Пн|Вт|Ср|Чт|Пт|Сб|Вс");
	int pos=0;
	for(int i=0;i<7;i++)
	{
		pos = line_week.find("|");
		lng_week[i] = line_week.left(pos);
		line_week.remove(0, pos+1);
	}
/// Lang Full Week
	//UTF("Пятница|Вторник|Среда|Четверг|Пятница|Суббота|Воскресенье");
	for(int i=0;i<7;i++)
	{
		pos = line_fweek.find("|");
		lng_fweek[i] = line_fweek.left(pos);
		line_fweek.remove(0, pos+1);
	}
/// Lang Month
	//UTF("Января|Февраля|Марта|Апреля|Мая|Июня|Июля|Августа|Сентября|Октября|Ноября|Декабря");
	for(int i=0;i<12;i++)
	{
		pos = line_month.find("|");
		lng_month[i] = line_month.left(pos);
		line_month.remove(0, pos+1);
	}
}

// --------------------------------------------------------
	
void kDesktopInfo::slotQCopReceived(const QCString &msg)
{
	/*if(msg=="idleScrFocusStatMsg")
	{
		qApp->processEvents();
		qApp->processEvents();
		usleep(25000);
		
		{
			Debug("desktop infocus");
			if(!my_focus)
			{
				//changeInfo();
				timer->start(timer_delay);
				my_focus=true;
			}
		}
		else
		{
			Debug("desktop outfocus");
			if(my_focus)
			{
				timer->stop();*/
				/*sleep(1);
				if(!QCopChannel::isRegistered("kLaunchInFocus"))
				{
					Debug("kLaunch outfocus");
					timer->stop();
				}*/
				/*my_focus=false;
			}
		}
	}*/
	if(msg=="quit")
	{
		timer->stop();
		clearBg();
		refresh();
		qApp->quit();//
		//Debug("kDesktopInfo::reseive mes");
		//if(!show_app)	showLaunch();
		//else			hideLaunch();
	}
}

double kDesktopInfo::tof(long size)
{
	double siz=size;
	return floor(siz/1024/1024*100)/100;///(double)
}

void kDesktopInfo::Debug(QString mes)
{
	if(debug)
		qDebug(mes);
}

int main(int argc, char **argv)
{
	QDaemonApp a(argc, argv);
	
	if (QCopChannel::isRegistered("kDesktopInfo"))
	{
		QCopChannel::send("kDesktopInfo", "quit");
		//system(kGetPath(argv[0])+"qcop_send kDesktopInfo quit");
		//Debug("kDesktopInfo::send mes");
	}
	else
	{
		QDaemonApp* a = new QDaemonApp(argc, argv);
		kDesktopInfo* dlg = new kDesktopInfo();
		a->setMainWidget(dlg);
		//dlg->show();
		a->exec();
	}
	return 0;
}


#include "kDesktopInfo.moc"
