//
// C++ Implementation: ZTaskMan
//
// Description: task manager
//
// Author: Ant-ON <prozanton@gmail.com>, (C) 2009
//
// Based on zSimpleGUI by BeZ
//
#include "zTaskMan.h"
#include "GUI_Define.h"
#include "BaseDlg.h"
//#include "ZAboutDlg.h"
#include "ZProcInfo.h"
#include "ZSettingDlg.h"
#include "lng.h"

#include "stdio.h"
#include <stdlib.h>
#include <unistd.h>
#include <qpixmap.h>

#include "QDaemonApp.h"

#include <qmap.h>
#include <qfile.h>

#include <dirent.h>
#include <fcntl.h>
#include "ZMsg.h"
#include "ZDefs.h"
#include <qevent.h>
#include <qdatastream.h>
#include <qtextstream.h>
#include <signal.h>
#include <sys/wait.h>
#include <ZConfig.h>
#include "exec.h"
#include <qregexp.h>
#include <qimage.h>
#include <ezxres.h>

extern QDaemonApp *app;
extern ZLng* lng;

ZTaskMan::ZTaskMan()
    : ZKbMainWidget ( ZHeader::TIME_TYPE, NULL, "ZMainWidget", WType_TopLevel )
{
	cop_hardkey = new QCopChannel("/hardkey/bc", this);
    connect( cop_hardkey, SIGNAL(received(const QCString&, const QByteArray&)), this, SLOT( signalReceived(const QCString&, const QByteArray&) ) );
	
	cop_menu = new QCopChannel("kTaskMan::Options", this);
    connect( cop_menu, SIGNAL(received(const QCString&, const QByteArray&)), this, SLOT( optionFunction(const QCString&) ) );
	
	toLog("Check run SDL app");
	/*if ( QCopChannel::isRegistered(SDL_LIB_CHENEL) )
	{
		QCopChannel::send( SDL_LIB_CHENEL, "hide()" );
		toLog("Send hide SDL lib",true, true);
	}*/
	
	ProgDir = getProgramDir();
	lng->ProgDir = ProgDir;
	pm_app.load(ProgDir+"/img/app.png");
	pm_java.load(ProgDir+"/img/java.png");
	pm_fav.load(ProgDir+"/img/run_app.png");
  
	CreateWindow();
	//app->showMainWidget ( this );
	
	channelStart = new QCopChannel(ZTASKMAN_START,this,"kTaskMan");
    CHECK_PTR(channelStart);
    connect( channelStart, SIGNAL(received(const QCString&, const QByteArray&)), this, SLOT( signalReceived(const QCString&, const QByteArray&) ) );
}

ZTaskMan::~ZTaskMan()
{
	delete lbProc;
    delete channelStart;
}

void ZTaskMan::signalReceived( const QCString &msg, const QByteArray &data)
{
	toLog("kTaskMan QCop Mes: "+msg,true,true);
	if ( msg == "show()" ) 
	{
		show();
		app->slotRaise();
		this->raise();
		this->setActiveWindow();
	} else 
	if ( msg == "newshow()" )
	{
		show();
		app->slotRaise();
		this->raise();
		this->setActiveWindow();
		buildProcList();
		//lbProc->showMenu();
	
		if (cfg_ShowNativIcon)
			nativIcon();
		if(java_name)
			kvmToName();
		
		showOptions();
		menu->hideOptionMenu();
		//QTimer::singleShot(100, lbProc, SLOT(repaint()));
		///lbProc->setFocus();
		//lbProc->showMenu();
	}
	
	if(msg=="keyMsg(int,int)" )
	{
		QDataStream stream(data, IO_ReadOnly);
		int key, type;
		stream >> key >> type;
		
		if(type==1)
		{
			if((key==4168 && key_note) || (key==4149 && key_voice) || (key==4169 && key_play) || (key==4172 && key_next) || (key==4171 && key_prew) || (key==4118 && key_vol_up) || (key==4119 && key_vol_dwn))
			{
				//QCopChannel::send( ZTASKMAN_START, "newshow()" );
				if(cfg_UpdateOnRaise)
					signalReceived("newshow()", NULL);
				else
					signalReceived("show()", NULL);
			}
		}
	}
}


void ZTaskMan::optionFunction( const QCString &msg)
{
	if ( msg == "procShow" )	menu_procShow();
	if ( msg == "procQuit" )	menu_procQuit();
	if ( msg == "procKill" )	menu_procKill();
	if ( msg == "procInfo" )	menu_procInfo();
	if ( msg == "updateList")	signalReceived("newshow()", NULL);
	if ( msg == "setting" )		menu_appSetting();
//	if ( msg == "about" )		about();
//	if ( msg == "quit" )		app->quit();
}

void ZTaskMan::createProcMenu()
{
	menu->clear();
	
	menu->addItem(lng->LNG_SHOW, "procShow", 2);
	menu->addItem(lng->LNG_QUIT, "procQuit", 2);
	menu->addItem(lng->LNG_KILL, "procKill", 2);
	menu->addItem(lng->LNG_INFO, "procInfo", 2);
	menu->addItem("----------", NULL, -1);
	menu->addItem(lng->LNG_UPDATE, "updateList", 2);
	menu->addItem("----------", NULL, -1);
	menu->addItem(lng->LNG_SETTING, "setting", 2);
//	menu->addItem(lng->LNG_ABOUT, "about", 2);
//	menu->addItem(lng->LNG_EXIT, "quit", 2);
	
	menu->hideOptionMenu();
}

void ZTaskMan::showOptions()
{
	menu->showOptionMenu();
}

void ZTaskMan::readCfg()
{
	ZConfig cfg(ProgDir+appConf);
	cfg_SendReaisePhone = cfg.readBoolEntry("General",	"SendReaisePhone", true);
	cfg_SendGoToIDLE = cfg.readBoolEntry("General",		"SendGoToIDLE", false);
	cfg_UpdateOnRaise = cfg.readBoolEntry("General",	"UpdateProcListOnRaise", false);
	lng->loadLngFile( cfg.readEntry("General",			"Lng", "Russian.lng") );
	//cfg_Lang = cfg.readEntry("General",				"Lng", "Russian.lng");
	cfg_FiltrProc = cfg.readBoolEntry("Task",			"FiltrProc", false);
	cfg_ShowJavaIcon = cfg.readBoolEntry("Task",		"ShowJavaIcon", false);
	cfg_ShowPid = cfg.readBoolEntry("Task",				"ShowPid", false);
	cfg_ShowNativIcon = cfg.readBoolEntry("Task",		"ShowNativIcon", false);
	cfg_ShowAppNoWindow = cfg.readBoolEntry("Task",		"ShowAppNoWindow", false);
	cfg_ShowIconOnRunApp = cfg.readBoolEntry("Task",	"ShowIconOnRunApp", false);
	
	key_note = cfg.readBoolEntry("Keys",	"Note",		false);
	key_voice = cfg.readBoolEntry("Keys",	"Voice",	false);
	key_play = cfg.readBoolEntry("Keys",	"Play",		false);
	key_next = cfg.readBoolEntry("Keys",	"Next",		false);
	key_prew = cfg.readBoolEntry("Keys",	"Prew",		false);
	key_vol_up = cfg.readBoolEntry("Keys",	"VolumeUp",	false);
	key_vol_dwn  = cfg.readBoolEntry("Keys","VolumeDown", false);
	cfg.flush();
}

void ZTaskMan::CreateWindow ()
{
	qApp->installEventFilter( this );
    	
 	//***************************** Config *****************************
	toLog("CreateWindow: Config");
	readCfg();
    	
	//************************** interface *****************************	
	toLog(QString("CreateWindow: interface"));
	setMainWidgetTitle ( "kTaskMan" );
  
	/*lbProc = new ZListBox ( QString ( "%16I%"+QString::number(SCREEN_WHIDTH - 26)+"M%20M" ), this, 0);
	lbProc->setFixedWidth ( SCREEN_WHIDTH );  */
	lbProc = new kListBox(300);

	setContentWidget ( lbProc );
	
	//lng->loadLngFile(cfg_Lang);
	
	//*********************** softkeys and menus ***********************
	toLog("CreateWindow: softkeys and menus");
  	
  	menu = new kOptionsMenu ( "kTaskMan::Options");
	createProcMenu();
	
	//**************************** SIGNAL ******************************
	toLog("CreateWindow: SIGNAL");
	connect(lbProc, SIGNAL (selected(int)),		this, SLOT (lbProcSel(int)) );
	
	//connect(lbProc, SIGNAL (rightSoftKey()),	this, SLOT (hide()) );
	//connect(lbProc, SIGNAL (leftSoftKey()),		this, SLOT (showOptions()) );
	//connect(lbProc, SIGNAL (ClearKey()),		this, SLOT (menu_procKill()) );
	//connect(lbProc, SIGNAL (Key0()),			this, SLOT (signalReceived("newshow()", NULL)) );
	
	connect(qApp, SIGNAL(signalRaise()),		this, SLOT(slotRaise()) );
	connect(qApp, SIGNAL(askReturnToIdle(int)),	this, SLOT(slotReturnToIdle(int)) );
	
	//************************* Build Lists ****************************
	toLog("CreateWindow: Build Lists");	
	buildProcList();
	if (cfg_ShowNativIcon)
		nativIcon();
	if(java_name)
		kvmToName();
	
	lbProc->setFocus();
	
	toLog(QString("CreateWindow: end"));
	
	//lng->loadLngFile(cfg_Lang);
}

void ZTaskMan::toLog(QString mes, bool r1, bool r2)
{
	/*if (r1)
		qDebug ( "================================" );
	if (mes != NULL)
		qDebug ( mes );
	if (r2)
		qDebug ( "================================" );*/
}

QString ZTaskMan::getProgramDir()
{
	ProgDir = QString ( qApp->argv() [0] ) ;
	int i = ProgDir.findRev ( "/" );
	ProgDir.remove ( i+1, ProgDir.length() - i );
	toLog(ProgDir, true, true);
	return ProgDir;
}

void ZTaskMan::about()
{
	/*lbProc->hideMenu();
	ZAboutDialog* aboutDlg = new ZAboutDialog();
	aboutDlg->exec();

	delete aboutDlg;
	aboutDlg = NULL;
	lbProc->showMenu();*/
}

void ZTaskMan::lbProcSel(int index)
{
	kListBoxItem listitem = lbProc->item(index);
	int pid=listitem.Pid;

	pidToTop(pid);
}

void ZTaskMan::slotRaise()
{
	toLog( "slot_Raise()", true);
	this->show();
	toLog( NULL, false, true);
}

void ZTaskMan::slotReturnToIdle( int reason )
{
	toLog( "slot_ReturnToIdle("+QString::number(reason)+")", true);
	this->hide();
	toLog( NULL, false, true);
}

void ZTaskMan::goToIdle()
{
	toLog("Miniumized all app");
	if (cfg_SendGoToIDLE)
	{
		//minimized java
		ZMsg msg( SYSTEM_CHANNEL, "ReturnToIdle(int)" );
		msg << (int)RETURN_IDLE_BY_FLIP;
		//msg << (int)RETURN_IDLE_BY_APP_TIMEOUT;
		msg.send();	
	}
	ZMsg msg( "EZX/Application/kvm", "ReturnToIdle(int)" );
	msg << (int)RETURN_IDLE_BY_END_KEY;
	msg.send();	
	if (cfg_SendReaisePhone)
	{
		//hide no minimized nativ
		sendMes( pidPhone, "raise()" );	
	}
	qApp->processEvents();
}

void ZTaskMan::pidToTop(int pid)
{ 
	//goToIdle();
	
	//sendMes( pidPhone, "raise()" );
	//sendMes( 0, "raise()" );
	//qApp->processEvents();
	//sendMes( pidAm, "clickAppIcon()" );
	
	toLog("App ["+QString::number(pid)+"] show");
	sendMes( pid, "raise()" ); 

	qApp->processEvents();
	//qApp->quit();
	hide();
}

void ZTaskMan::sendMes(int pid, QString mes)
{  
  QCopChannel::send( QCString("EZX/Application/"+QString::number(pid)), QCString ( mes ), QByteArray() );
}

void ZTaskMan::pidQuit(int pid)
{ 
	sendMes( pid, "quickQuit()" );      
}

bool ZTaskMan::pidKill( int pid)
{
    if ( ::kill( pid, SIGKILL ) < 0 )
    {
		toLog( "Error when kill a pid ["+QString::number(pid) );
		return false;
    }
    toLog( "Killed a pid ["+QString::number(pid) );
    return true;
}

QString ZTaskMan::readFileLine(const QString & fileName)
{
    QFile f(fileName);
    QString s;

    if ( f.open( IO_ReadOnly ) )
    {
        QTextStream t( &f );

        if ( !t.eof() )
	{
            s = t.readLine();
        }

        f.close();
    }

    return s;

}

bool ZTaskMan::processIsRuning(pid_t pid)
{
    QString fn;
    QString s = readFileLine(fn.sprintf("/proc/%d/cmdline",pid));

    if(s.isEmpty())
    {
        return false;
    }
    else
    {
        return true;
    }
}

void ZTaskMan::menu_procShow()
{
	int i = lbProc->currentItem();
	if (i == -1) 
	{
		return;
	}
	lbProcSel(i);
}

void ZTaskMan::menu_procQuit()
{
	int i = lbProc->currentItem();
	if (i == -1) 
	{
		return;
	}
	kListBoxItem listitem = lbProc->item ( i );
	int pid=listitem.Pid;

	pidQuit( pid );
	
    for ( int i = 0; i < 50; i++ )
    {
        if ( !processIsRuning( pid ) )
        {
			lbProc->removeItem ( lbProc->currentItem() );
			lbProc->setCurrentItem(0);
            break;
        }

        qApp->processEvents();
        //
        // 0.2*25 is 5 seconds
        //
        usleep( 200 );
    }
}

void ZTaskMan::menu_procKill()
{	
	int i = lbProc->currentItem();
	if (i == -1) 
	{
		return;
	}
	kListBoxItem listitem = lbProc->item ( i );
	int pid=listitem.Pid;

	pidKill( pid );
	
    for ( int i = 0; i < 25; i++ )
    {
        if ( !processIsRuning( pid ) )
        {
			lbProc->removeItem ( lbProc->currentItem() );
			lbProc->setCurrentItem(0);
            break;
        }

        qApp->processEvents();
        //
        // 0.2*25 is 5 seconds
        //
        usleep( 200 );
    }	
}

void ZTaskMan::menu_procInfo()
{
	int i = lbProc->currentItem();
	lbProc->hideMenu();
	if (i == -1) 
	{
		return;
	}
	kListBoxItem listitem = lbProc->item ( i );
	int pid=listitem.Pid;

	ZProcInfoDlg* infoDlg = new ZProcInfoDlg( pid );
	infoDlg->exec();

	delete infoDlg;
	infoDlg = NULL;
	lbProc->showMenu();
}

void ZTaskMan::menu_appSetting()
{
	lbProc->hideMenu();
	//hide();
	toLog("menu_appSetting: Start");
	
	ZAppSetting* settingDlg = new ZAppSetting( );
	settingDlg->exec();

	//if ( settingDlg->res() == 1 )
	//{
		lbProc->clear();
		buildProcList();
		if (cfg_ShowNativIcon)
			nativIcon();
		if(java_name)
			kvmToName();
		
		lbProc->setFocus();
		
		readCfg();
		createProcMenu();
	//}

	delete settingDlg;
	settingDlg = NULL;
	
	toLog("menu_appSetting: End");
	lbProc->showMenu();
	//show();
}

bool ZTaskMan::addedProc(int pid)
{
	int i;
	for (i=0;i<lbProc->count();i++)
	{
		kListBoxItem listitem = lbProc->item ( i );
		if ( listitem.Pid == pid )
		{
			return true;
			break;
		}
	}
	return false;
}

QString ZTaskMan::extractExecName(QString nameProc)
{
	if (nameProc.find(".sh") != -1 || nameProc.find(".lin") != -1)
	{
		QString str = nameProc;
		int i = str.findRev ("/");
		if (i!=-1)
			nameProc=str.mid(i+1, str.length()-i);
		i = nameProc.find(" ");
		if (i==1)
			return nameProc;
		else
			return nameProc.mid(0, i+1);
	}
	else
	{
		nameProc.remove(nameProc.find(" "), nameProc.length() );
		if (nameProc.find(" ")==-1 || nameProc.find(" ") > nameProc.find("/"))
		{
			while ( nameProc.find("/") > -1 )
			{
				nameProc.remove(0,nameProc.find("/")+1 );
			}	
		}		
		return nameProc;
	}
}

bool ZTaskMan::procFilter(QString nameProc)
{
	if ( cfg_FiltrProc == 0 )
	{
		return 1;		
	}
	QString noShowInList;
	QFile file(ProgDir+"filter.ini");
	if( file.open(IO_ReadOnly) )		
	{
		QTextStream stream( &file );
		//stream.setEncoding ( QTextStream::UnicodeUTF8 );
		noShowInList=stream.readLine();
	}
	QString shortName;
	shortName = extractExecName( nameProc );
	if (noShowInList.find( "!" + shortName + "!" ) > -1)
	{
		return 0;	
	} 
	
	return 1;
}

bool ZTaskMan::procFilterSh(QString nameProc)
{
	/*if ( nameProc.find(".sh") != -1 )
	{
		return 1;
	}	
	
	if ( nameProc.find(".lin") != -1 )
	{
		return 1;
	}
	return 0;*/
	
	return 1;
}

QString ZTaskMan::getJavaFolderInSD()
{
	toLog("getJavaFolderInSD()");
	static DIR *dir;
	struct dirent *entry;
	char *name;
	QString qname;		

	dir = opendir("/mmc/mmca1/.system");
	if(!dir)
	{
		toLog("Can't open SD Card", true, true);
	} else	
	{
		for(;;) 
		{	
			if((entry = readdir(dir)) == NULL) 
			{
				closedir(dir);
				dir = 0;
				break;
			}	
			name = entry->d_name;
			
			
			qname = QString(name);
			
			if (qname.find("java_")>-1)
			{
				toLog("JavaFolderInSD: "+ QString(qname));
				return qname;
			}
		}
	}
	toLog("JavaFolderInSD: not found");
	return NULL;	
}

void ZTaskMan::kvmToName()
{
	//********************* Build list of java name ********************
	toLog("Build list of java name");
	typedef QMap<QString, QString> mapJavaAppName;
	mapJavaAppName mapJava;
	
	QString sRes = QString::null;
	QString jname;
	QString id;
	
	QString javaInSD = "java";//getJavaFolderInSD();
	
	QFile file;
	
	int SDCount=0;
	
	if ( javaInSD != NULL )
	{
		toLog("java name: get name on SD");
		file.setName( "/mmc/mmca1/.system/"+ javaInSD +"/JAMInfo/index_card1" );
		if (file.exists())
		{
			file.open( IO_ReadOnly );
			
			while ( ! file.atEnd() ) 
			{
				file.readLine(sRes,256);
				jname=sRes;
				jname.remove(0,7);
				jname.remove(jname.find("||"),jname.length() );
				id=sRes;
				id.remove(id.find("||"),id.length() );
				while ( id.find("0")==0 )
					id.remove(0,1);
				mapJava[ id ] ="[kvm] "+jname;	
				++SDCount;	
			}
			file.close();
		} else
		{
			toLog("java name: get name on SD - no exist DB");
		}
	}
	
	file.setName( "/ezxlocal/.system/java/JAMInfo/index" );
	toLog("java name: get name on Phone");
	if (file.exists())
	{
		file.open( IO_ReadOnly );
		
		while ( ! file.atEnd() ) 
		{
			file.readLine(sRes,256);
			jname=sRes;
			jname.remove(0,7);
			jname.remove(jname.find("||"),jname.length() );
			id=sRes;
			id.remove(id.find("||"),id.length() );
			while ( id.find("0")==0 )
				id.remove(0,1);		
			mapJava[ id ] = "[kvm] "+jname;	
		}
		file.close();	
	} else
	{
		toLog("java name: get name on Phone - no exist DB");
	}
	//********************* Replace name ********************
	toLog("java name: Replace name");
	int i;
	int n;
	QString nameProc;
	for (i=0;i<lbProc->count();i++)
	{
		kListBoxItem listitem = lbProc->item ( i );
		nameProc = listitem.Name;
		if (nameProc.find("kvm") > -1)
		{
			n=0;
			for ( mapJavaAppName::Iterator it = mapJava.begin( ); it != mapJava.end( ); it++) 
			{ 
				++n;
				if ( nameProc.find(" "+it.key()) > -1 )
				{
					jname="";
					if (cfg_ShowJavaIcon)
					{
						if (n > SDCount)
						{
							jname = "/ezxlocal/.system/java/DownloadApps/MIDlet00"+it.key()+"/Files/MIDlet-1.png";
							if ( !QFile::exists(jname) && QFile::exists("/ezxlocal/download/mystuff/.system/java/SystemApps/MIDlet00"+it.key()+"/Files/MIDlet-1.png") )
							{
								jname = "/ezxlocal/.system/java/SystemApps/MIDlet00"+it.key()+"/Files/MIDlet-1.png";
							}
						} else
						{
							jname = "/mmc/mmca1/.system/"+ javaInSD +"/DownloadApps/MIDlet"+it.key()+"/Files/MIDlet-1.png" ;
						}
					} 
					/*if ( !QFile::exists(jname) )
					{				
        				jname = ProgDir + QString ( "/img/java.png" );
					}*/
					if ( QFile::exists(jname) )
					{
						QPixmap pm;
						toLog("LoadIMG: "+jname);
						pm.load( jname );
						if ( pm.height() > 19 || pm.width() > 19 )
						{
							toLog("Mashtab Image");
							QImage image;
							image = pm.convertToImage();
							image = image.smoothScale(19, 19);
							pm.convertFromImage(image);	
						}	
						listitem.Icon=pm;
					}
					else
						listitem.Icon=pm_java;
        			listitem.Name=QString(it.data());	
					lbProc->changeItem(i, listitem.Name, listitem.Icon, listitem.Pid/*, int type*/);
					break;						
				}		
			}
		}
	}
	
	lbProc->setCurrentItem(0);
}

void ZTaskMan::nativIcon()
{
	//********************* Build list of java name ********************
	toLog("Build list of icon");
	typedef QMap<QString, BDRegistry> mapIconNativ;
	mapIconNativ mapIcon;
	
	QString ico;
	QString nname;
	QString dir;
	
	ZConfig confCARD ( CARD_REGISTRY, false );
	QStringList grouplist;
	confCARD.getGroupsKeyList ( grouplist );
	for ( QStringList::Iterator it = grouplist.begin(); it != grouplist.end(); ++it )
	{	 
		nname = confCARD.readEntry ( *it, "Exec", "" );
		dir=QString("/mmc/mmca1/.system/QTDownLoad/%1/").arg(confCARD.readEntry ( *it, "Directory", "" ));
		ico = confCARD.readEntry ( *it, "BigIcon", "" );
		if (ico=="")
		{
			ico = confCARD.readEntry ( *it, "Icon", "" );
		}
		if (confCARD.readEntry ( *it, "JavaId", "" )=="" && nname!="" && ico!="")
		{
			BDRegistry app;
			app.setIcon(dir+ico);
			app.setName(confCARD.readEntry ( *it, "Name", "" ));
			app.setT(false);
			mapIcon[ nname ] = app;
		}
	}	
	
	ZConfig confUSER ( INSTALLED_DB, false );
	confUSER.getGroupsKeyList ( grouplist );
	for ( QStringList::Iterator it = grouplist.begin(); it != grouplist.end(); ++it )
	{	 
		nname = confUSER.readEntry ( *it, "Exec", "" );
		dir=QString("/ezxlocal/.system/QTDownLoad/%1/").arg(confCARD.readEntry ( *it, "Directory", "" ));
		ico = confUSER.readEntry ( *it, "BigIcon", "" );
		if (ico=="")
		{
			ico = confUSER.readEntry ( *it, "Icon", "" );
		}
		if (confUSER.readEntry ( *it, "JavaId", "" )=="" && nname!="" && ico!="")
		{
			BDRegistry app;
			app.setIcon(dir+ico);
			app.setName(confUSER.readEntry ( *it, "Name", "" ));
			app.setT(false);
			mapIcon[ nname ] = app;
		}
	}
	
	ZConfig confSYS ( SYS_REGISTRY, false );
	confSYS.getGroupsKeyList ( grouplist );
	for ( QStringList::Iterator it = grouplist.begin(); it != grouplist.end(); ++it )
	{	 
		nname = confSYS.readEntry ( *it, "Exec", "" );
		dir="";
		ico = confSYS.readEntry ( *it, "BigIcon", "" );
		if (ico=="")
		{
			ico = confSYS.readEntry ( *it, "Icon", "" );
		}
		if (confSYS.readEntry ( *it, "JavaId", "" )=="" && nname!="" && ico!="")
		{
			BDRegistry app;
			app.setIcon(dir+ico);
			app.setName(confSYS.readEntry ( *it, "Name", "" ));
			app.setT(true);
			mapIcon[ nname ] = app;
		}
	}
	
	//********************* Repleace name ********************
	toLog("Nativ: Repleace icon");
	int i;
	QString nameProc;
	for (i=0;i<lbProc->count();i++)
	{
		kListBoxItem listitem = lbProc->item ( i );
		nameProc = listitem.Name;
		if (nameProc.find("kvm") == -1)
		{
			for ( mapIconNativ::Iterator it = mapIcon.begin( ); it != mapIcon.end( ); it++) 
			{ 
				nname=it.key();
				if ( (!it.data().t() && nname.find(nameProc) != -1) || (it.data().icon() && nname == nameProc) )
				{
					QPixmap pm;
					ico = it.data().icon();
					nname = it.data().name();
					if (ico.find(".g") != -1)
					{
						RES_ICON_Reader res;
						pm = res.getIcon(ico,true);
					} else
					{
						nname=ico;
						if (QFile::exists(ico) )
							pm.load( ico );
						else
							pm=pm_app;
					}
        			if ( pm.height() > 19 || pm.width() > 19 )
					{
	                    QImage image;
                        image = pm.convertToImage();
                        image = image.smoothScale(19, 19);
                        pm.convertFromImage(image);	
                    }
                    if (nname=="")
                    {
        				listitem.Icon=pm;
        			} else
        			{
        				listitem.Name=nname;
        				listitem.Icon=pm;
					} 
					lbProc->changeItem(i, listitem.Name, listitem.Icon, listitem.Pid/*, int type*/);
					break;	
				}
			}
		}
	}
}

void ZTaskMan::buildProcList()
{
	//************************* Get runing proc ************************
	toLog("Get runing proc");
	lbProc->clear();
	lbProc->showPid=cfg_ShowPid;
	pidPhone = 0;
	java_name=false;

	static DIR *dir;
	struct dirent *entry;
	char *name;
	int n;
	char buf[1024];
	FILE *fp;
	int pid;
	char *cmd;
	
	QString jname;	

	dir = opendir("/proc");
	if(!dir)
		toLog("Can't open /proc", true, true);
		
	for(;;) 
	{
		cmd = 0;
		
		if((entry = readdir(dir)) == NULL) 
		{
			closedir(dir);
			dir = 0;
			toLog("Get runing proc: End");
			break;
		}
		name = entry->d_name;
		if (!(*name >= '0' && *name <= '9'))
			continue;

		pid = atoi(name);
		
		if((fp = fopen(QCString("/proc/"+QString::number(pid)+"/cmdline"), "r")) == NULL)
			continue;
		if((n=fread(buf, 1, sizeof(buf)-1, fp)) > 0) 
		{
			if(buf[n-1]=='\n')
				buf[--n] = 0;
			name = buf;
			while(n) 
			{
				if(((unsigned char)*name) < ' ')
					*name = ' ';
				name++;
				n--;
			}
			*name = 0;
			if(buf[0])
				cmd = strdup(buf);
		}
		fclose(fp);
		
		if ( QCopChannel::isRegistered(QCString(APP_CHANNEL_PREF + QString::number(pid))) )
		{
			// add proces in list
			if (cmd != 0 && cmd[0] != 0 )
			{
				
				jname = QString(cmd);
				jname.stripWhiteSpace();
				
				//Save pid of phone proces for minimized all app
				if ( pidPhone == 0 && jname.find("SYSqtapp/phone/phone") > -1 )
				{
					pidPhone = pid;
					toLog("pidPhone="+QString::number(pidPhone),true,true);			
				}				
				
				if ( procFilter(jname) )
				{
					kListBoxItem listitem;
					if ( (jname.find("kvm") > -1) )
					{
						/*QPixmap pm;
						pm.load(ProgDir + QString ( "/img/java.png" ));*/
						listitem.Icon=pm_java;
						listitem.Name=QString(jname);
						listitem.Pid=pid;						
						lbProc->addItem(listitem);  
						java_name=true;
					} else
					{
						/*QPixmap pm;
						pm.load(ProgDir + QString ( "/img/app.png" ));*/
						listitem.Icon=pm_app;
						listitem.Name=QString(extractExecName(QString(cmd)));
						listitem.Pid=pid;						
						lbProc->addItem(listitem);
					}
				}
			}
		}
		else
		if (cfg_ShowAppNoWindow)
		{
			if (procFilterSh(QString(cmd)) && !addedProc(pid) && !extractExecName(QString(cmd)).isNull())
			{
				kListBoxItem listitem;
				QPixmap pm;
				pm.load(ProgDir + QString ( "/img/sh.png" ));
				listitem.Icon=pm;
				listitem.Name=extractExecName(QString(cmd));
				listitem.Pid=pid;	
				lbProc->addItem(listitem);
			}			
		}
	}			
}

bool ZTaskMan::eventFilter(QObject* o, QEvent* pEvent)
{ 
    if (QEvent::KeyPress == pEvent->type())
    {
		QKeyEvent *pKey = (QKeyEvent*)pEvent;

		if(pKey->key() == Z6KEY_RED)
		{
			//qApp->quit();
			menu->hideOptionMenu();
			hide();
			return true;
		}
		if(pKey->key() == 0x1032 && lbProc->show_app)
		{
			menu_procKill();
			return true;
		}
		if(pKey->key() == 0x1038 && lbProc->show_app)
		{
			showOptions();
			return true;
		}
		if(pKey->key() == 0x103a && lbProc->show_app)
		{
			menu->hideOptionMenu();
			hide();
			return true;
		}
		if(pKey->key() == 48 && lbProc->show_app)
		{
			signalReceived("newshow()", NULL);
			return true;
		}
	}
    return QWidget::eventFilter( o, pEvent ); 
}
